#include <iostream>
#include <QtCore>
#include <QtDebug>
#include "httpget.h"

TileRequest::TileRequest(const QString &key, const QPoint &pos, int z) :
        myKey(key), myPos(pos), myZ(z)
{}

HttpGet::HttpGet(QObject *parent) :
        QObject(parent)
{
    myHttp = new QHttp();
    connect(myHttp, SIGNAL(done(bool)), this, SLOT(httpDone(bool)));
    connect(myHttp, SIGNAL(stateChanged(int)), this, SLOT(changeState(int)));
}

HttpGet::~HttpGet() {
    qDebug()<<"destructor "<<myRequest.key();
    myHttp->abort();
    delete myHttp;
    qDebug()<<"complete";
}

bool HttpGet::getFile(const QUrl &url, const TileRequest& request) {
    myUrl = url;
    myRequest = request;
    if (!myUrl.isValid()) {
        std::cerr<<"Error: Invalid URL"<<std::endl;
        return false;
    }
    if (myUrl.path().isEmpty()) {
        std::cerr<<"Error: Path is empty"<<std::endl;
        return false;
    }
    myHttp->setHost(myUrl.host(), myUrl.port(80));
    myHttp->get(myUrl.path(), &myBuffer);
    myHttp->close();
    return true;
}

void HttpGet::httpDone(bool error) {
    qDebug()<<"done: "<<myRequest.key()<<" size: "<<myBuffer.data().size();
    if (error) {
        std::cerr<<qPrintable(myHttp->errorString())<<std::endl;
    }
    emit done(this, error);
}

void HttpGet::changeState(int state) {
    //qDebug()<<"state: "<<state<<" "<<myRequest.key()<<" size: "<<myBuffer.data().size();
    if (state == QHttp::Unconnected)
        emit finished(this);
}

void HttpGet::abort() {
    myHttp->abort();
    myBuffer.close();
}
