#ifndef HTTPGET_H
#define HTTPGET_H

#include <QObject>
#include <QPoint>
#include <QString>
#include <QBuffer>
#include <QHttp>
#include <QUrl>

class TileRequest {
private:
    QString myKey;
    QPoint myPos;
    int myZ;
public:
    TileRequest() {}
    TileRequest(const QString& key, const QPoint& pos, int z);
    const QString& key() { return myKey; }
    const QPoint& pos() { return myPos; }
    const int& z() { return myZ; }
    bool operator==(TileRequest req) { return this->key() == req.key(); }
};

class HttpGet : public QObject
{
    Q_OBJECT
private:
    QUrl myUrl;
    QHttp *myHttp;
    QBuffer myBuffer;
    TileRequest myRequest;
public:
    HttpGet(QObject *parent = 0);
    virtual ~HttpGet();
    bool getFile(const QUrl& url, const TileRequest& request);
    const QByteArray& getData() const { return myBuffer.data(); }
    void close() { myBuffer.close(); }
    const TileRequest& request() { return myRequest; }
signals:
    void done(HttpGet* getter, bool error);
    void finished(HttpGet*);
private slots:
    void httpDone(bool);
    void changeState(int);
public slots:
    void abort();
};

#endif // HTTPGET_H
