#include <QtGui>
#include <QtDebug>
#include "mainwindow.h"
#include "model.h"
#include "mapscene.h"
#include "mapview.h"
#include "tilesizedlg.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    model = new Model();
    scene = new MapScene(model);
    scene->setSceneRect(0, 0, 256*model->width(), 256*model->height());
    view = new MapView(scene);
    createActions();
    createMenuBar();
    createToolBar();
    createStatusBar();
    printer = new QPrinter();
    QGridLayout *mainLayout = new QGridLayout();
    mainLayout->addWidget(view, 1, 1);
    QHBoxLayout *northLayout = new QHBoxLayout();
    northLayout->addStretch(1);
    QToolButton *bAddNorth = new QToolButton();
    bAddNorth->setDefaultAction(addNorthAction);
    northLayout->addWidget(bAddNorth);
    QToolButton *bMoveNorth = new QToolButton();
    bMoveNorth->setDefaultAction(moveNorthAction);
    northLayout->addWidget(bMoveNorth);
    QToolButton *bDelNorth = new QToolButton();
    bDelNorth->setDefaultAction(delNorthAction);
    northLayout->addWidget(bDelNorth);
    northLayout->addStretch(1);
    mainLayout->addLayout(northLayout, 0, 0, 1, 3);
    QVBoxLayout *westLayout = new QVBoxLayout();
    westLayout->addStretch(1);
    QToolButton *bAddWest = new QToolButton();
    bAddWest->setDefaultAction(addWestAction);
    westLayout->addWidget(bAddWest);
    QToolButton *bMoveWest = new QToolButton();
    bMoveWest->setDefaultAction(moveWestAction);
    westLayout->addWidget(bMoveWest);
    QToolButton *bDelWest = new QToolButton();
    bDelWest->setDefaultAction(delWestAction);
    westLayout->addWidget(bDelWest);
    westLayout->addStretch(1);
    mainLayout->addLayout(westLayout, 1, 0);
    QVBoxLayout *eastLayout = new QVBoxLayout();
    eastLayout->addStretch(1);
    QToolButton *bAddEast = new QToolButton();
    bAddEast->setDefaultAction(addEastAction);
    eastLayout->addWidget(bAddEast);
    QToolButton *bMoveEast = new QToolButton();
    bMoveEast->setDefaultAction(moveEastAction);
    eastLayout->addWidget(bMoveEast);
    QToolButton *bDelEast = new QToolButton();
    bDelEast->setDefaultAction(delEastAction);
    eastLayout->addWidget(bDelEast);
    eastLayout->addStretch(1);
    mainLayout->addLayout(eastLayout, 1, 2);
    QHBoxLayout *southLayout = new QHBoxLayout();
    southLayout->addStretch(1);
    QToolButton *bAddSouth = new QToolButton();
    bAddSouth->setDefaultAction(addSouthAction);
    southLayout->addWidget(bAddSouth);
    QToolButton *bMoveSouth = new QToolButton();
    bMoveSouth->setDefaultAction(moveSouthAction);
    southLayout->addWidget(bMoveSouth);
    QToolButton *bDelSouth = new QToolButton();
    bDelSouth->setDefaultAction(delSouthAction);
    southLayout->addWidget(bDelSouth);
    southLayout->addStretch(1);
    mainLayout->addLayout(southLayout, 2, 0, 1, 3);
    QWidget *center = new QWidget();
    center->setLayout(mainLayout);
    setCentralWidget(center);
    scene->redraw();
    connect(model, SIGNAL(mapChanged()), this, SLOT(updateModelStatus()));
    connect(view, SIGNAL(mouseTile(int,int)), this, SLOT(updateMouseTile(int, int)));
    updateModelStatus();
    //getImage("http://mapserver.mapy.cz/turist/5_7800000_8000000");
}

MainWindow::~MainWindow()
{

}

void MainWindow::createActions() {
    tileSizeAction = new QAction(tr("&Tile size..."), this);
    printAction = new QAction(tr("&Print..."), this);
    printAction->setIcon(QIcon(":/icons/printer.png"));
    redrawAction = new QAction(tr("&Redraw"), this);
    redrawAction->setIcon(QIcon(":/icons/arrow_refresh.png"));
    QActionGroup *functionActionGroup = new QActionGroup(this);
    posAction = new QAction(QIcon(":/icons/information.png"), tr("Tile Position"), this);
    posAction->setCheckable(true);
    posAction->setChecked(true);
    functionActionGroup->addAction(posAction);
    zoomInAction = new QAction(QIcon(":/icons/zoom_in.png"), tr("Zoom in"), this);
    zoomInAction->setCheckable(true);
    functionActionGroup->addAction(zoomInAction);
    zoomOutAction = new QAction(QIcon(":/icons/zoom_out.png"), tr("Zoom out"), this);
    zoomOutAction->setCheckable(true);
    functionActionGroup->addAction(zoomOutAction);
    QActionGroup *typeActionGroup = new QActionGroup(this);
    typeActionGroup->setExclusive(true);
    turistMapAction = new QAction(tr("&Tourist"), this);
    turistMapAction->setCheckable(true);
    turistMapAction->setChecked(true);
    typeActionGroup->addAction(turistMapAction);
    armyMapAction = new QAction(tr("&Military"), this);
    armyMapAction->setCheckable(true);
    typeActionGroup->addAction(armyMapAction);
    ophotoMapAction = new QAction(tr("&Orthophoto"), this);
    ophotoMapAction->setCheckable(true);
    turistOvlAction = new QAction(tr("&Hiking"), this);
    turistOvlAction->setCheckable(true);
    hybridOvlAction = new QAction(tr("&Labels"), this);
    hybridOvlAction->setCheckable(true);
    reliefOvlAction = new QAction(tr("&Contour"), this);
    reliefOvlAction->setCheckable(true);
    typeActionGroup->addAction(ophotoMapAction);
    addNorthAction = new QAction(tr("Extend to north"), this);
    addNorthAction->setIcon(QIcon(":/icons/add.png"));
    addEastAction = new QAction(tr("Extend to east"), this);
    addEastAction->setIcon(QIcon(":/icons/add.png"));
    addSouthAction = new QAction(tr("Extend to south"), this);
    addSouthAction->setIcon(QIcon(":/icons/add.png"));
    addWestAction = new QAction(tr("Extend to west"), this);
    addWestAction->setIcon(QIcon(":/icons/add.png"));
    delNorthAction = new QAction(tr("Remove from north"), this);
    delNorthAction->setIcon(QIcon(":/icons/delete.png"));
    delEastAction = new QAction(tr("Remove from east"), this);
    delEastAction->setIcon((QIcon(":/icons/delete.png")));
    delSouthAction = new QAction(tr("Remove from south"), this);
    delSouthAction->setIcon((QIcon(":/icons/delete.png")));
    delWestAction = new QAction(tr("Remove from west"), this);
    delWestAction->setIcon(QIcon(":/icons/delete.png"));
    moveNorthAction = new QAction(tr("Move to north"), this);
    moveNorthAction->setIcon(QIcon(":/icons/arrow_up.png"));
    moveEastAction = new QAction(tr("Move to east"), this);
    moveEastAction->setIcon(QIcon(":/icons/arrow_right.png"));
    moveSouthAction = new QAction(tr("Move to south"), this);
    moveSouthAction->setIcon(QIcon(":/icons/arrow_down.png"));
    moveWestAction = new QAction(tr("Move to west"), this);
    moveWestAction->setIcon(QIcon(":/icons/arrow_left.png"));
    connect(tileSizeAction, SIGNAL(triggered()), this, SLOT(setTileSize()));
    connect(printAction, SIGNAL(triggered()), this, SLOT(print()));
    connect(redrawAction, SIGNAL(triggered()), scene, SLOT(redraw()));
    connect(posAction, SIGNAL(triggered()), view, SLOT(setShowFunction()));
    connect(zoomInAction, SIGNAL(triggered()), view, SLOT(setZoomInFunction()));
    connect(zoomOutAction, SIGNAL(triggered()), view, SLOT(setZoomOutFunction()));
    connect(turistMapAction, SIGNAL(triggered()), this, SLOT(setTuristMap()));
    connect(armyMapAction, SIGNAL(triggered()), this, SLOT(setArmyMap()));
    connect(ophotoMapAction, SIGNAL(triggered()), this, SLOT(setOphotoMap()));
    connect(turistOvlAction, SIGNAL(triggered()), this, SLOT(changeOverlays()));
    connect(hybridOvlAction, SIGNAL(triggered()), this, SLOT(changeOverlays()));
    connect(reliefOvlAction, SIGNAL(triggered()), this, SLOT(changeOverlays()));
    connect(addNorthAction, SIGNAL(triggered()), this, SLOT(addNorth()));
    connect(addEastAction, SIGNAL(triggered()), this, SLOT(addEast()));
    connect(addSouthAction, SIGNAL(triggered()), this, SLOT(addSouth()));
    connect(addWestAction, SIGNAL(triggered()), this, SLOT(addWest()));
    connect(delNorthAction, SIGNAL(triggered()), this, SLOT(delNorth()));
    connect(delEastAction, SIGNAL(triggered()), this, SLOT(delEast()));
    connect(delSouthAction, SIGNAL(triggered()), this, SLOT(delSouth()));
    connect(delWestAction, SIGNAL(triggered()), this, SLOT(delWest()));
    connect(moveNorthAction, SIGNAL(triggered()), this, SLOT(moveNorth()));
    connect(moveEastAction, SIGNAL(triggered()), this, SLOT(moveEast()));
    connect(moveSouthAction, SIGNAL(triggered()), this, SLOT(moveSouth()));
    connect(moveWestAction, SIGNAL(triggered()), this, SLOT(moveWest()));
}

void MainWindow::createMenuBar() {
    QMenu *mFile = menuBar()->addMenu(tr("&File"));
    mFile->addAction(tileSizeAction);
    mFile->addAction(printAction);
    QMenu *mView = menuBar()->addMenu(tr("&View"));
    mView->addAction(redrawAction);
    QMenu *mType = mView->addMenu(tr("&Type"));
    mType->addAction(turistMapAction);
    mType->addAction(armyMapAction);
    mType->addAction(ophotoMapAction);
    QMenu *mOverlays = mView->addMenu(tr("&Overlays"));
    mOverlays->addAction(turistOvlAction);
    mOverlays->addAction(hybridOvlAction);
    mOverlays->addAction(reliefOvlAction);
}

void MainWindow::createToolBar() {
    QToolBar *toolBar = addToolBar(tr("Tools"));
    toolBar->addAction(printAction);
    toolBar->addSeparator();
    toolBar->addAction(redrawAction);
    toolBar->addAction(posAction);
    toolBar->addAction(zoomInAction);
    toolBar->addAction(zoomOutAction);
    toolBar->addSeparator();
    toolBar->addAction(turistOvlAction);
    toolBar->addAction(hybridOvlAction);
    toolBar->addAction(reliefOvlAction);
}

void MainWindow::createStatusBar() {
    lModelStatus = new QLabel();
    statusBar()->addWidget(lModelStatus);
    lPos = new QLabel();
    statusBar()->addWidget(lPos);
}

void MainWindow::showImage(HttpGet *getter) {
    qDebug()<<"showImage";
    QPixmap pixmap;
    bool res = pixmap.loadFromData(getter->getData());
    qDebug()<<"pixmap "<<res;
    if (res) {
        //label->setPixmap(pixmap);
        delete getter;
        getter = 0;
    }
}

void MainWindow::setTileSize() {
    TileSizeDlg dlg;
    dlg.setTileSize(model->tileSize());
    if (dlg.exec() == QDialog::Accepted) {
        model->setTileSize(dlg.tileSize());
    }
}

void MainWindow::print() {
    double tilesize = model->tileSize(); // mm
    //printer.setOutputFileName("xxx.pdf");
    //printer.setOutputFormat(QPrinter::PdfFormat);
    QPrintDialog dlg(printer, this);
    if (dlg.exec()) {
        qDebug()<<printer->pageRect(QPrinter::Millimeter)<<" "<<printer->pageRect();
        int dpix = printer->logicalDpiX();
        int dpiy = printer->logicalDpiY();
        qDebug()<<"width: "<<printer->width()<<" "<<dpix;
        double tw = tilesize/25.4*dpix;
        double th = tilesize/25.4*dpiy;
        int maxw = (int)(printer->width()/tw);
        int maxh = (int)(printer->height()/th);
        int pagesx = (model->width()-1)/maxw +1;
        int pagesy = (model->height()-1)/maxh+1;
        if (QMessageBox::information(this, tr("Pages"), tr("Printout needs %1 pages").arg(pagesx*pagesy),
                                     QMessageBox::Ok| QMessageBox::Cancel, QMessageBox::Ok)
            != QMessageBox::Ok) {
            return;
        }
        QPainter painter(printer);
        for (int ix = 0; ix < pagesx; ix++) {
            for (int iy = 0; iy < pagesy; iy++) {
                if (ix+iy > 0) {
                    printer->newPage();
                }
                scene->render(&painter, QRectF(0, 0, tw*maxw, th*maxh),
                              QRectF(ix*maxw*256, iy*maxh*256, maxw*256, maxh*256));
            }
        }
    }
}

void MainWindow::setTuristMap() {
    model->setType("turist");
}

void MainWindow::setArmyMap() {
    qDebug()<<"setArmyMap";
    model->setType("army2");
}

void MainWindow::setOphotoMap() {
    model->setType("ophoto");
}

void MainWindow::changeOverlays() {
    QList<Overlay> ovls;
    if (turistOvlAction->isChecked()) {
        ovls.append(Overlay("ttur", 10));
    }
    if (hybridOvlAction->isChecked()) {
        ovls.append(Overlay("hybrid", 5));
    }
    if (reliefOvlAction->isChecked()) {
        ovls.append(Overlay("relief-l", 3));
    }
    model->changeOverlays(ovls);
}

void MainWindow::addNorth() {
    model->changeSize(1, 0, 0, 0);
}

void MainWindow::addEast() {
    model->changeSize(0, 1, 0, 0);
}

void MainWindow::addSouth() {
    model->changeSize(0, 0, 1, 0);
}

void MainWindow::addWest() {
    model->changeSize(0, 0, 0, 1);
}

void MainWindow::delNorth() {
    model->changeSize(-1, 0, 0, 0);
}

void MainWindow::delEast() {
    model->changeSize(0, -1, 0, 0);
}

void MainWindow::delSouth() {
    model->changeSize(0, 0, -1, 0);
}

void MainWindow::delWest() {
    model->changeSize(0, 0, 0, -1);
}

void MainWindow::moveNorth() {
    model->changeSize(1, 0, -1, 0);
}

void MainWindow::moveEast() {
    model->changeSize(0, 1, 0, -1);
}

void MainWindow::moveSouth() {
    model->changeSize(-1, 0, 1, 0);
}

void MainWindow::moveWest() {
    model->changeSize(0, -1, 0, 1);
}

void MainWindow::updateModelStatus() {
    lModelStatus->setText(tr("width: %1 height: %2 zoom: %3")
                          .arg(model->width()).arg(model->height()).arg(model->zoom()));
}

void MainWindow::updateMouseTile(int ix, int iy) {
    int x0 = model->x();
    int y0 = model->y();
    iy = model->height()-1-iy;
    int z = model->zoom();
    int offs = model->offset();
    lPos->setText(tr("Tilename: %1_%2_%3").arg(z).arg(x0+offs*ix, 0, 16).arg(y0+offs*iy, 0, 16));
}
