#ifndef MAPSCENE_H
#define MAPSCENE_H

#include <QList>
#include <QString>
#include <QGraphicsScene>
#include "httpget.h"
#include "model.h"

class HttpGet;
class QProgressDialog;
class QPixmap;

class PixmapEntry
{
private:
    QString myKey;
    QPixmap *myPixmap;
public:
    PixmapEntry(const QString& key, QPixmap *pixmap) :
            myKey(key), myPixmap(pixmap)
    {}
    const QString& key() { return myKey; }
    QPixmap *pixmap() { return myPixmap; }
};

class MapScene : public QGraphicsScene
{
    Q_OBJECT
private:
    Model *myModel;
    HttpGet getter;
    QList<PixmapEntry> pixmaps;
    QList<TileRequest> requests;
    QProgressDialog *progressDlg;
    QPixmap *getPixmap(const QString& key);
    void addPixmap(QPixmap *pixmap, int ix, int iy, int z);
    void getNextTile();
    void redrawLayer(const QString& key, int z);
public:
    MapScene(Model *model, QObject *parent = 0);
    Model *model() { return myModel; }
private slots:
    void tileLoaded(HttpGet *getter, bool error);
    void cancelRequests();
public slots:
    void redraw();
};

#endif // MAPSCENE_H
