#include <QtGui>
#include <QtDebug>
#include "model.h"
#include "mapscene.h"
#include "mapview.h"
#include "viewfunction.h"

MapView::MapView(QGraphicsScene *scene) :
        QGraphicsView(scene)
{
    function = new ShowFunction(this);
}

void MapView::mousePressEvent(QMouseEvent *event) {
    QPointF pos = mapToScene(event->pos());
    int ix = (int)pos.x()/256;
    int iy = (int)pos.y()/256;
    qDebug()<<"Mousepos: "<<ix<<" "<<iy;
    if (event->button() == Qt::LeftButton) {
        function->b1(ix, iy);
    }
    else if (event->button() == Qt::RightButton) {
        mouseX0 = pos.x();
        mouseY0 = pos.y();
    }
}

void MapView::mouseMoveEvent(QMouseEvent *event) {
    QPointF pos = mapToScene(event->pos());
    int ix = (int)pos.x()/256;
    int iy = (int)pos.y()/256;
    if ((event->buttons() & Qt::RightButton) != 0) {
        qDebug()<<"motion "<<pos.x()-mouseX0<<" "<<pos.y()-mouseY0;
        QScrollBar *sb = verticalScrollBar();
        qDebug()<<"vsb "<<sb->value();
        sb->setValue(sb->value()-pos.y()+mouseY0);
        sb = horizontalScrollBar();
        qDebug()<<"hsb "<<sb->value();
        sb->setValue(sb->value()-pos.x()+mouseX0);
    }
    emit mouseTile(ix, iy);
}

void MapView::wheelEvent(QWheelEvent *event) {
    QPointF pos = mapToScene(event->pos());
    int ix = (int)pos.x()/256;
    int iy = (int)pos.y()/256;
    qDebug()<<"Mousepos: "<<ix<<" "<<iy<<" "<<event->delta();
    if (event->delta() > 0) {
        zoomIn(ix, iy);
    }
    else if (event->delta() < 0) {
        zoomOut(ix, iy);
    }
}

void MapView::zoomIn(int ix, int iy) {
    Model *model = static_cast<MapScene*>(scene())->model();
    model->zoomIn(ix, iy);
}

void MapView::zoomOut(int ix, int iy) {
    Model *model = static_cast<MapScene*>(scene())->model();
    model->zoomOut(ix, iy);
}



void MapView::setShowFunction() {
    delete function;
    function = new ShowFunction(this);
}

void MapView::setZoomInFunction() {
    delete function;
    function = new ZoomInFunction(this);
}

void MapView::setZoomOutFunction() {
    delete function;
    function = new ZoomOutFunction(this);
}
