#include <QtDebug>
#include "model.h"

Overlay::Overlay(const QString &key, int z):
        myKey(key), myZ(z)
{}

Model::Model()
{
    myZoom = 6;
    myX = 0x7800000;
    myY = 0x7c00000;
    myWidth = 5;
    myHeight = 3;
    myType = QString("turist");
    myTileSize = 40;
}

int Model::offset() {
    return 1<<(28-myZoom);
}

void Model::zoomIn(int ix, int iy) {
    if (myZoom >= 17) {
        return;
    }
    iy = myHeight-1-iy;
    qDebug()<<"posOld: "<<myX<<", "<<myY<<" delta "<<ix<<", "<<iy;
    int offs = offset();
    int x = myX + ix*offs;
    int y = myY + iy*offs;
    myZoom++;
    offs /= 2;
    myX = x - (myWidth-2)/2*offs;
    myY = y - (myHeight-2)/2*offs;
    qDebug()<<"posNew: "<<myX<<", "<<myY;
    emit mapChanged();
}

void Model::zoomOut(int ix, int iy) {
    if (myZoom < 6) {
        return;
    }
    iy = myHeight-1-iy;
    int offs = offset();
    int x = myX + ix*offs;
    int y = myY + iy*offs;
    myZoom--;
    offs *= 2;
    myX = x - ((myWidth-2)/2)*offs;
    myY = y - ((myHeight-2)/2)*offs;
    myX = (myX/offs)*offs;
    myY = (myY/offs)*offs;
    emit mapChanged();
}

void Model::changeSize(int north, int east, int south, int west) {
    if (myWidth+west+east < 2 || myHeight+north+south < 2)
        return;
    int offs = offset();
    myX -= west*offs;
    myY -= south*offs;
    myWidth += west+east;
    myHeight += north+south;
    emit mapChanged();
}

void Model::changeOverlays(const QList<Overlay>&  overlays) {
    myOverlays.clear();
    foreach(Overlay ovl, overlays) {
        myOverlays.append(ovl);
    }
    emit mapChanged();
}
